@@ -1,7 +1,7 @@
 /***************************************************************************
 **                                                                        **
 **  Polyphone, a soundfont editor                                         **
-**  Copyright (C) 2013-2019 Davy Triponney                                **
+**  Copyright (C) 2013-2024 Davy Triponney                                **
 **                                                                        **
 **  This program is free software: you can redistribute it and/or modify  **
 **  it under the terms of the GNU General Public License as published by  **
@@ -18,7 +18,7 @@
 **                                                                        **
 ****************************************************************************
 **           Author: Davy Triponney                                       **
-**  Website/Contact: https://www.polyphone-soundfonts.com                 **
+**  Website/Contact: https://www.polyphone.io                             **
 **             Date: 01.01.2013                                           **
 ***************************************************************************/
 
@@ -34,8 +34,8 @@
 #include <QDebug>
 #include "soundfontdetails.h"
 
-//const QString RepositoryManager::BASE_URL = "http://localhost/polyphone-soundfonts/api/soundfonts/";
-const QString RepositoryManager::BASE_URL = "https://www.polyphone-soundfonts.com/api/soundfonts/";
+//const QString RepositoryManager::BASE_URL = "http://www.polyphone.local/api/soundfonts/";
+const QString RepositoryManager::BASE_URL = "https://www.polyphone.io/api/soundfonts/";
 RepositoryManager * RepositoryManager::s_instance = nullptr;
 
 RepositoryManager * RepositoryManager::getInstance()
@@ -171,7 +171,7 @@ void RepositoryManager::fillLicenseLabelAndLink()
     _licenseLabels["personal_use_only"] = tr("personal use, don't distribute a modified version");
 
     // Links
-    _licenseUrl = "https://www.polyphone-soundfonts.com/license#";
+    _licenseUrl = "https://www.polyphone.io/en/licenses#";
 }
 
 void RepositoryManager::initialize()
@@ -182,7 +182,7 @@ void RepositoryManager::initialize()
         delete _soundfontInfos.take(key);
 
     // Notify the list is downloading
-    emit(initializing());
+    emit initializing();
 
     // Download the list
     _urlReaderJsonList->download();
@@ -201,7 +201,7 @@ void RepositoryManager::soundfontListAvailable(QString error)
     // Notify that the list is ready
     if (!error.isEmpty())
         qCritical() << "RepositoryManager::soundfontListAvailable() - received error" << error;
-    emit(ready(error));
+    emit ready(error);
 }
 
 QString RepositoryManager::loadSoundfontInfo()
@@ -460,7 +460,7 @@ QString RepositoryManager::getLicenseLabel(QString licenseKey)
 
 QString RepositoryManager::getLicenseLink(QString licenseKey)
 {
-    return _licenseUrl + licenseKey;
+    return _licenseUrl + licenseKey.replace('_', '-').replace("no-restrictions", "no-more-restrictions");
 }
 
 void RepositoryManager::openSoundfont(int soundfontId, bool daily)
@@ -495,5 +495,5 @@ void RepositoryManager::openSoundfont(int soundfontId, bool daily)
     }
 
     // Ask to open the page
-    emit(openSoundfont(soundfontId));
+    emit openSoundfont(soundfontId);
 }
